#!/usr/bin/perl

use strict;
use warnings;
use Getopt::Lucid qw( :all );

# Define command line options
my @params = (
        Param("hostname|h")->required,
        Param("state|s")->required,
        Param("type|t")->required,
	Param("contacts|c"),
);

# Parse & objectify command line options
my $options;
eval { $options = Getopt::Lucid->getopt( \@params ) };

# Catch any exceptions thrown by Getopt::Lucid
if ($@) {
        print "$@\n" && print_usage();
}

my $hostname = $options->get_hostname;  # Hostname supplied by Nagios
my $state = $options->get_state;        # 0=OK, 1=WARNING, 2=CRITICAL, 3=UNKNOWN
my $type = $options->get_type;          # HARD or SOFT. SOFT states are still pending verification

our $recipients = 'daelic@gmail.com';

my $username = getUserID($hostname);

my $command = "/usr/bin/uname -a";
report_findings("step 1","."); # Progress Test 1
my %result = run_command($command, $hostname, $username);
report_findings("step 2","."); # Progress Test 2

my $ostring = $result{'out'};

report_findings("Test Script Result","$ostring\nHost: $hostname\nState: $state\nType: $type\n");

sub run_command {
        my $command = $_[0];
        my $host = $_[1];
        my $username = $_[2];

        my %result = SSHexec($command, $hostname, $username, '/home/nagios/.ssh/id_dsa');
        if ($result{'err'}) {
                my $subject = "Automated refreshtac failed on $hostname";
                my $message = "Error when attempting SSH connection to $hostname:\n";
                $message .= "  ".$result{'err'}."\n";
                report_findings($subject, $message, $recipients);
                exit;
        }
        return %result;
}

sub SSHexec {
        use Net::SSH::Perl;

        my $CMD = $_[0];
        my $HOST = $_[1];
        my $USR = $_[2];
	my $identity = $_[3];
        my %RESULT;
	my $ssh;

        if (pingHost($HOST)) {
                %RESULT = ("err", "Connectivity Failure");
                return %RESULT;
        }

	if ($identity) {
		$ssh = Net::SSH::Perl->new($HOST, options => [ "protocol '2,1'" ], identity_files => ["$identity" ]);
	} else {
		$ssh = Net::SSH::Perl->new($HOST, options => [ "protocol '2,1'" ]);
	}

        unless ($ssh->login($USR)) {
                %RESULT = ("err", "Authentication Failure");
                return %RESULT;
        }
        my ($out,$err,$exit) = $ssh->cmd($CMD);
        %RESULT = ("out", $out,
                        "err", $err,
                        "exit", $exit);
        return %RESULT;
}

sub report_findings {
        my $subject = $_[0];
        my $message = $_[1];
        sendMail($message, $subject, $recipients);
}

sub pingHost {
        use Net::Ping;

        my $host = $_[0];
        my $PING = Net::Ping->new("tcp", 2);
        $PING->{port_num} = 22;
        if ($PING->ping($host)) {
                return 0;
        }
        return 1;
}

sub getUserID {
        my $HOST = $_[0];
        my @USERCHECK = split(/\s+/,`grep "^$HOST" /etc/sshuserids`);
        my $USERID = $USERCHECK[1];
        return $USERID unless ($USERID ne "gtacuser");
        return 'root';
}

sub sendMail {
        use Mail::Sendmail;
        use Sys::Hostname;

        my $host = hostname;
        my $sender;

        my $message = $_[0];
        my $subject = $_[1];
        my $recipients = $_[2];
        $sender = $_[3];
        $sender = "root\@$host" unless ($sender);

        my %mail = (
                from    => $sender,
                to      => $recipients,
                subject => $subject,
                body    => $message,
        );
        sendmail(%mail) || return $Mail::Sendmail::error;
        return 0;
}

sub print_usage {
        use File::Basename;
        my $basename = basename($0);

        my $message = "$basename - Nagios Event Handler with remote execution\n";
        $message .= "\nAll options are required: \n";
        $message .= "\t--hostname -h | host to check\n";
        $message .= "\t--state -s | 0,1,2,3\n";
        $message .= "\t--type -t | HARD or SOFT\n";
        $message .= "\t--contacts -c | Must be a comma separated string\n";

        print "$message\n";
        exit;
}

